import * as tables from './tables.es6';
import {uint8Inc, uint8Dec, uint16Inc, text2Uint8Array, uint8Array2Text} from '../util/util.es6';
import getDebugger from '../util/debug.es6';

export class Reciter {
  constructor (options) {
    const opts = options;
    this.getOptions = () => opts;
    this.debug = getDebugger(opts.debug);
    this._A = 0;
    this._X = 0;
    this._Y = 0;
    this._outpos = null; //output position for phonemes
    this._mem57 = null;
    this._mem58 = null;
    this._mem60 = null;
    this._mem61 = null;
    this._ruleTablePos = null; // memory position of current rule
    this._mem64 = null; // position of '=' or current character
    this._closeBrace = null; // position of ')'
    this._openBrace = null; // position of '('
    this.inputtemp = null;
  }

  get A () { return this._A }
  set A (v) { this._A = v & 0xFF }
  get X () { return this._X }
  set X (v) { this._X = v & 0xFF }
  get Y () { return this._Y }
  set Y (v) { this._Y = v & 0xFF }
  get mem56 () { return this._outpos }
  set mem56 (v) { this._outpos = v & 0xFF }
  get mem57 () { return this._mem57 }
  set mem57 (v) { this._mem57 = v & 0xFF }
  get mem58 () { return this._mem58 }
  set mem58 (v) { this._mem58 = v & 0xFF }
  get mem60 () { return this._mem60 }
  set mem60 (v) { this._mem60 = v & 0xFF }
  get mem61 () { return this._mem61 }
  set mem61 (v) { this._mem61 = v & 0xFF }
  get mem62 () { return this._ruleTablePos }
  set mem62 (v) { this._ruleTablePos = v & 0xFFFF }
  get mem64 () { return this._mem64 }
  set mem64 (v) { this._mem64 = v & 0xFF }
  get mem65 () { return this._closeBrace }
  set mem65 (v) { this._closeBrace = v & 0xFF }
  get mem66 () { return this._openBrace }
  set mem66 (v) { this._openBrace = v & 0xFF }

  getRuleContent (offset) {
    let i = 1;
    let ch = 0;
    let rule = '';
    do {
      ch = this.GetRuleByte(offset, i);
      if ((ch&127) === 61) { // '='
        rule += ' -> ';
      } else {
        rule += String.fromCharCode(ch & 127);
      }
      i++;
    } while ((ch&128) === 0);
    return rule;
  }

  /* Retrieve flags for character at mem59-1 */
  Code37055 (npos, mask) {
    this.X = npos;
    return tables.tab36376[this.inputtemp[X]] & mask;
  }

  match (str) {
    let ch;
    for (ch = 0; ch<str.length;ch++) {
      this.A = this.inputtemp[this.X++];
      if (this.A !== (str.charCodeAt(ch) && 0xFF)) {
        return 0;
      }
    }
    return 1;
  }

  GetRuleByte (mem62, Y) {
    if (!this.breaker) {this.breaker=0}
    if (this.breaker++ === 2000) {
      throw new Error('Break stuff!');
    }

    let address = mem62;
    if (mem62 >= 37541) {
      address -= 37541;
      return tables.rules2[address+Y];
    }
    address -= 32000;

    return tables.rules[address+Y];
  }

  handle_ch2(ch, mem) {
    this.X = mem;
    let tmp = tables.tab36376[this.inputtemp[mem]];
    if (ch === 32) { // ' '
      if (tmp & 128) {
        return 1;
      }
    } else if (ch === 35) { // '#'
      if (!(tmp & 64)) {
        return 1;
      }
    } else if (ch === 46) { // '.'
      if (!(tmp & 8)) {
        return 1;
      }
    } else if (ch === 94) { // '^'
      if (!(tmp & 32)) {
        return 1;
      }
    } else {
      return -1;
    }
    return 0;
  }

  handle_ch(A, mem) {
    this.X = mem;
    let tmp = tables.tab36376[this.inputtemp[this.X]];
    if (A === 32) { // ' '
      if ((tmp & 128) !== 0) return 1;
    } else if (A === 35) { // '#'
      if ((tmp & 64) === 0) return 1;
    } else if (A === 46) { // '.'
      if((tmp & 8) === 0) return 1;
    } else if (A === 38) { // '&'
      if((tmp & 16) === 0) {
        if (this.inputtemp[this.X] !== 72) return 1;
        ++this.X;
      }
    } else if (A === 94) { // '^'
      if ((tmp & 32) === 0) return 1;
    } else if (A === 43) { // '+'
      this.X = mem;
      A = this.inputtemp[this.X];
      if ((A !== 69) && (A !== 73) && (A !== 89)) return 1;
    } else return -1;
    return 0;
  }

  /**
   * Convert the text to a phoneme string.
   * @return {boolean}
   */
  TextToPhonemes (text) {
    while (text.length < 256) {
      text += ' ';
    }

    this.input = text2Uint8Array(text);
    this.input[this.input.length - 1] = 135; // '['

    this.inputtemp = new Uint8Array(256);

    // secure copy of input
    // because input will be overwritten by phonemes
    this.X = 0;
    this.inputtemp[0] = 32; // ' '
    do {
      this.A = this.input[this.X] & 127;
      if (this.A >= 112) {
        this.A = this.A & 95;
      } else if (this.A >= 96) {
        this.A = this.A & 79;
      }
      this.inputtemp[++this.X] = this.A;
    } while (this.X < 255);
    this.inputtemp[255] = 27;

    this.mem56 = this.mem61 = 255;
    this.mem57 = null;

    let result = this.pos36554();
    this.debug.log(uint8Array2Text(this.input));
    this.debug.log(uint8Array2Text(this.inputtemp));
    if (result) {
      return result;
    }
    return this.input;
  }

  pos36554 () {
    console.log('pos36554');
    while (1) {
      while(1) {
        this.X = ++this.mem61;
        this.mem64 = this.inputtemp[this.X];
        if (this.mem64 === 91) { // '['
          this.X = ++this.mem56;
          this.input[this.X] = 155;
          return 1;
        }

        if (this.mem64 !== 46) {
          break;
        }
        this.X++;
        this.A = tables.tab36376[this.inputtemp[this.X]] & 1;
        if (this.A !== 0) break;
        this.mem56++;
        this.X = this.mem56;
        this.A = '.';
        this.input[this.X] = 46; // '.'
      }
      this.mem57 = tables.tab36376[this.mem64];
      if((this.mem57 & 2) !== 0) {
        this.mem62 = 37541;
        return this.pos36700();
      }

      if(this.mem57 !== 0) break;
      this.inputtemp[this.X] = 32; // ' '
      this.X = ++this.mem56;
      if (this.X > 120) {
        this.input[this.X] = 155;
        return 1;
      }
      this.input[this.X] = 32;
    }

    if(!(this.mem57 & 128)) return 0;

    // go to the right rules for this character.
    this.X = this.mem64 - 65; // 'A'
    this.mem62 = tables.tab37489[this.X] | (tables.tab37515[this.X]<<8);

    return this.pos36700()
  }

  pos36700 () {
    console.log('pos36700');
    // find next rule
    while ((this.GetRuleByte(++this.mem62, 0) & 128) === 0);
    let Y = 0;
    while(this.GetRuleByte(this.mem62, ++Y) !== 40); // '('
    this.mem66 = Y;
    while(this.GetRuleByte(this.mem62, ++Y) !== 41); // ')'
    this.mem65 = Y;
    while((this.GetRuleByte(this.mem62, ++Y) & 127) !== 61); // '='
    this.mem64 = Y;

    this.mem60 = this.X = this.mem61;
    // compare the string within the bracket
    Y = this.mem66 + 1;

    while(1) {
      if (this.GetRuleByte(this.mem62, Y) !== this.inputtemp[this.X]) {
        return this.pos36700();
      }
      if(++Y === this.mem65) {
        break;
      }
      this.mem60 = ++this.X;
    }

    // the string in the bracket is correct
    let mem59 = this.mem61;

    while(1) {
      while(1) {
        this.mem66--;
        this.mem57 = this.GetRuleByte(this.mem62, this.mem66);
        if ((this.mem57 & 128) !== 0) {
          this.mem58 = this.mem60;
          return this.pos37184();
        }
        this.X = this.mem57 & 127;
        if ((tables.tab36376[this.X] & 128) === 0) {
          break;
        }
        if (this.inputtemp[mem59 - 1] !== this.mem57) {
          return this.pos36700();
        }
        --mem59;
      }

      let ch = this.mem57;
      let r = this.handle_ch2(ch, mem59-1);
      if (r === -1) {
        switch (ch) {
          case 38: // '&'
            if (!this.Code37055(mem59-1,16)) {
              if (this.inputtemp[this.X] !== 72) { // 'H'
                r = 1;
              } else {
                this.A = this.inputtemp[--this.X];
                if ((this.A !== 67) && (this.A !== 83)) { // 'C', 'S'
                  r = 1;
                }
              }
            }
            break;

          case 64: // '@'
            if(!this.Code37055(mem59-1,4)) {
              this.A = this.inputtemp[this.X];
              if (this.A !== 72) {
                r = 1;
              }
              if ((this.A !== 84) && (this.A !== 67) && (this.A !== 83)) { // 'T', 'C', 'S'
                r = 1;
              }
            }
            break;
          case 43: // '+'
            this.X = mem59;
            this.A = this.inputtemp[--this.X];
            if ((this.A !== 69) && (this.A !== 73) && (this.A !== 89)) { // 'E', 'I', 'Y'
              r = 1;
            }
            break;
          case 58: // ':'
            while (this.Code37055(mem59-1,32)) {
              --mem59;
            }
            continue;
          default:
            return 0;
        }
      }

      if (r === 1) {
        return this.pos36700();
      }

      mem59 = this.X;
    }

    do {
      this.X = this.mem58 + 1;
      if (this.inputtemp[this.X] === 69) { // 'E'
        if((tables.tab36376[this.inputtemp[this.X + 1]] & 128) !== 0) {
          this.A = this.inputtemp[++this.X];
          if (this.A === 76) { // 'L'
            if (this.inputtemp[++this.X] !== 89) { // Y
              return this.pos36700();
            }
          } else if ((this.A !== 82) && (this.A !== 83) && (this.A !== 68) && !this.match('FUL')) { // 'R' 'S' 'D'
            return this.pos36700();
          }
        }
      } else {
        if (!this.match('ING')) return this.pos36700();
        this.mem58 = X;
      }

      let tmp = this.pos37184();
      console.log(tmp);
    } while (this.A === 37); // '%'
    return 0;
  }

  pos37184() {
    console.log('pos37184');
    // X;
    let r = 0;
    do {
      let mem57;
      while (1) {
        let Y = this.mem65 + 1;
        if (Y  === this.mem64) {
          this.mem61 = this.mem60;

          this.debug.log(this.getRuleContent(this.mem62));

          while (1) {
            mem57 = this.A = this.GetRuleByte(this.mem62, Y);
            this.A = this.A & 127;
            if (this.A !== 61) { // '='
              this.input[++this.mem56] = this.A;
            }
            if ((mem57 & 128) !== 0) {
              return this.pos36554();
            }
            Y++;
          }
        }
        this.mem65 = Y;
        mem57 = this.GetRuleByte(this.mem62, Y);
        if((tables.tab36376[mem57] & 128) === 0){
          break;
        }
        if (this.inputtemp[this.mem58+1] !== mem57) {
          r = 1;
          break;
        }
        ++this.mem58;
      }

      if (r === 0) {
        this.A = mem57;
        if (this.A === 64) { // '@'
          if(this.Code37055(this.mem58+1, 4) === 0) {
            this.A = this.inputtemp[this.X];
            if ((this.A !== 82) && (this.A !== 84) &&
              (this.A !== 67) && (this.A !== 83)) r = 1;
          } else {
            r = -2;
          }
        } else if (this.A === 58) { // ':'
          while (this.Code37055(this.mem58+1, 32)) {
            this.mem58 = this.X;
          }
          r = -2;
        } else r = this.handle_ch(this.A, this.mem58+1);
      }

      if (r === 1) {
        return this.pos36700();
      }
      if (r === -2) {
        r = 0;
        continue;
      }
      if (r === 0) {
        this.mem58 = this.X;
      }
    } while (r === 0);
  }
}
















function PrintPhonemes(phonemeindex, phonemeLength, stress) {
    let i = 0;
    console.log("===========================================\n");

    console.log("Internal Phoneme presentation:\n\n");
    console.log(" idx    phoneme  length  stress\n");
    console.log("------------------------------\n");

    while((phonemeindex[i] !== 255) && (i < 255))
    {
        if (phonemeindex[i] < 81)
        {
            console.log(" %3i      %s%s      %3i       %i\n",
                phonemeindex[i],
                String.fromCharCode(signInputTable1[phonemeindex[i]]),
                String.fromCharCode(signInputTable2[phonemeindex[i]]),
                phonemeLength[i],
                stress[i]
            );
        } else
        {
            console.log(" %3i      ??      %3i       %i\n", phonemeindex[i], phonemeLength[i], stress[i]);
        }
        i++;
    }
    console.log("===========================================\n");
    console.log("\n");
}

export default Reciter;
