import * as tables from './tables-c.es6';
import {text2Uint8Array} from '../util/util.es6';

function uint8Inc (val) {
  return (++val) &0xFF;
}
function uint8Dec (val) {
  return (--val) &0xFF;
}
function uint16Inc (val) {
  return (++val) &0xFFFF;
}

/**
 * Global helper to retrieve a rule byte.
 *
 * @param {Number} ruleIndex
 * @param {Number} offset
 * @return Number
 */
function GetRuleByte(ruleIndex, offset) {
  let address = ruleIndex;
  if (ruleIndex >= 37541) {
    address -= 37541;
    return tables.rules2[address+offset];
  }
  address -= 32000;

  return tables.rules[address+offset];
}

export class Reciter {
  constructor(options) {
    const opts = options || {};
    this.getOptions = () => opts;
  }

  /**
   * Convert the text to a phoneme string.
   * @return {boolean}
   */
  TextToPhonemes (text) {
    // find '('
    function findOpeningBracket (ruleIndex, offset) {
      let tmp;
      while (true)
      {
        if (40 === (tmp = GetRuleByte(ruleIndex, offset))) { // '('
          break;
        }
        offset = uint8Inc(offset);
      }
      return offset;
    }

    // find ')'
    function findClosingBracket(ruleIndex, offset) {
      let tmp = offset;
      do {
        tmp = uint8Inc(tmp);
      } while(GetRuleByte(ruleIndex, tmp) !== 41); // ')'
      return tmp;
    }

    // find '='
    function searchEqualSign (ruleIndex, offset) {
      let tmp = offset;
      do {
        tmp = uint8Inc(tmp);
      } while ((GetRuleByte(ruleIndex, tmp) & 127) !== 61); // '='
      return tmp;
    }

    function currentRuleMatches() {
      let offset = uint8Inc(currentRule.bracketOpen);
      //pos36759:
      while(1)
      {
        mem57 = inputtemp[mem60];
        if (GetRuleByte(ruleIndex, offset) !== mem57) {
          return false;
        }
        offset = uint8Inc(offset);
        if (offset === currentRule.bracketClose) {
          return true;
        }
        mem60 = uint8Inc(mem60);
      }
    }

    // no vars before here!

    while (text.length < 256) {
      text += ' ';
    }
    const input = text2Uint8Array(text);
    input[input.length - 1] = 135; // '['

    let inputtemp = new Uint8Array(256);
    // secure copy of input
    // because input will be overwritten by phonemes
    (function cloneInput(buffer, clone) {
      let X = 1;
      let Y = 0;
      clone[0] = 32;
      do {
        let value = buffer[Y] & 127;
        if ( value >= 112) {
          value = value & 95;
        }
        else if ( value >= 96) {
          value = value & 79;
        }

        clone[X] = value;
        X = uint8Inc(X);
        Y = uint8Inc(Y);
      } while (Y !== 255);
    })(input, inputtemp);

    let X;

    let outbufferPos = 0;      //output position for phonemes
    let mem57 = 0;
    let mem58 = 0;
    let mem59 = 0;
    let mem60 = 0;
    let inputIndex = 0;
    let ruleIndex = 0;     // memory position of current rule

    let currentRule = {
      bracketOpen: 0,  // position of '('
      bracketClose: 0, // position of ')'
      ruleCurrentPos: 0,       // position of '=' or current character
    };

    X = 255;
    inputtemp[255] = 27;
    inputIndex = 255;
    let state = pos36554;
    let counter = 0;
    outbufferPos = 255;
    while (true) {
      if (counter++ > 1000) {
        return input;
      }
      if (typeof state === 'function') {
        state = state.apply();
      } else {
        if (0 !== state) {
          // throw new Error('Something strange in the neighborhood: ' + state);
          return false;
        }
        return input;
      }
    }

    function pos36554 () {
      while (true) {
        while(true) {
          inputIndex = uint8Inc(inputIndex);
          let pos = inputIndex;
          currentRule.ruleCurrentPos = inputtemp[pos];
          if (inputtemp[pos] === 91) // '['
          {
            outbufferPos = uint8Inc(outbufferPos);
            pos = outbufferPos;
            input[pos] = 155;
            // goto pos36542;
            //			Code39771(); 	//Code39777();
            return 1;
          }

          //pos36579:
          if (inputtemp[pos] !== 46) { // '.'
            break;
          }
          if((tables.charFlags[inputtemp[uint8Inc(pos)]] & 1) !== 0) {
            break;
          }
          outbufferPos = uint8Inc(outbufferPos);
          input[outbufferPos] = 46; // '.'
        } //while

        // pos36607:
        mem57 = tables.charFlags[currentRule.ruleCurrentPos];
        if ((mem57 & 2) !== 0)
        {
          ruleIndex = 37541;
          return goNextRule;
        }
        //pos36630:
        if (mem57 !== 0) {
          if((mem57 & 128) === 0)
          {
            //36683: BRK
            return 0;
          }

          // go to the right rules for this character.
          let pos = currentRule.ruleCurrentPos - 65; // 'A'
          ruleIndex = (tables.tab37489[pos] | (tables.tab37515[pos]<<8)) & 0xFFFF;
          return goNextRule;
        }
        let tmp = 32;
        inputtemp[outbufferPos] = 32; // ' '
        outbufferPos = uint8Inc(outbufferPos);
        // FIXME: might bear a problem.
        if (outbufferPos > 120) {
            input[outbufferPos] = 155;
            tmp = inputIndex;
            // mem36653 = tmp;
            //	mem29 = A; // not used
            //	Code36538(); das ist eigentlich
            return 1;
            //Code39771();
            //go on if there is more input ???
            inputIndex = mem36653;
            // return pos36550;
        }
        input[outbufferPos] = tmp;
        // -----
        //36653 is unknown. Contains position
      }
    }
    function goNextRule () {
      // -------------------------------------
      // go to next rule
      // -------------------------------------
      // find next rule delimited by 0x80.
      do {
        ruleIndex = uint16Inc(ruleIndex);
      } while ((GetRuleByte(ruleIndex, 0) & 128) === 0);

      currentRule.bracketOpen = findOpeningBracket(ruleIndex, 1);
      currentRule.bracketClose = findClosingBracket(ruleIndex, currentRule.bracketOpen);
      currentRule.ruleCurrentPos = searchEqualSign (ruleIndex, currentRule.bracketClose);

      mem60 = inputIndex;
      // compare the string within the bracket
      if (!currentRuleMatches()) {
        return goNextRule;
      }

      // the string in the bracket is correct
      //pos36787:
      mem59 = inputIndex;
      return pos36791;
    }
    function pos36791 () {
      while(1) {
        let tmp;
        currentRule.bracketOpen = uint8Dec(currentRule.bracketOpen);
        tmp = GetRuleByte(ruleIndex, currentRule.bracketOpen);
        mem57 = tmp;
        //36800: BPL 36805
        if ((tmp & 128) !== 0) {
          tmp = mem60;
          mem58 = tmp;
          return pos37184;
        }
        tmp = tables.charFlags[(tmp & 127)] & 128;
        if (tmp === 0) {
          break;
        }
        let pos = uint8Dec(mem59);
        tmp = inputtemp[pos];
        if (tmp !== mem57) {
          return goNextRule;
        }
        mem59 = pos;
      }
      //pos36833:
      if (mem57 === 32) { // ' '
        let tmp = uint8Dec(mem59);
        if ((tables.charFlags[inputtemp[tmp]] & 128) !== 0) {
          return goNextRule;
        }
        mem59 = tmp;
        return pos36791;
      }
      if (mem57 === 35) { // '#'
        let tmp = uint8Dec(mem59);
        if ((tables.charFlags[inputtemp[tmp]] & 64) !== 0) {
          mem59 = tmp;
          return pos36791;
        }
        return goNextRule;
      }
      if (mem57 === 46) { // '.'
        let tmp = uint8Dec(mem59);
        if ((tables.charFlags[inputtemp[tmp]] & 8) === 0) {
          return goNextRule;
        }
        mem59 = tmp;
        return pos36791;
      }
      if (mem57 === 38) { // '&'
        let pos = uint8Dec(mem59);
        if((tables.charFlags[inputtemp[pos]] & 16) !== 0) {
          mem59 = pos;
          return pos36791;
        }
        if (inputtemp[pos] !== 72) {
          return goNextRule;
        }
        pos = uint8Dec(pos);
        if ((inputtemp[pos] === 67) || (inputtemp[pos] === 83)) {
          mem59 = pos;
          return pos36791;
        }
        return goNextRule;
      }
      if (mem57 === 64) { // '@'
        let pos = uint8Dec(mem59);
        if ((tables.charFlags[inputtemp[pos]] & 4) !== 0) {
          mem59 = pos;
          return pos36791;
        }
        if (inputtemp[pos] !== 72) { // 'H'
          return goNextRule;
        }
        if ((inputtemp[pos] !== 84) && (inputtemp[pos] !== 67) && (inputtemp[pos] !== 83)) { // 'T', 'C', 'S'
          return goNextRule;
        }
        mem59 = pos;
        return pos36791;
      }
      if (mem57 === 94) { // '^'
        let pos = uint8Dec(mem59);
        if ((tables.charFlags[inputtemp[pos]] & 32) === 0) {
          return goNextRule;
        }
        mem59 = pos;
        return pos36791;
      }
      if (mem57 === 43) { // '+'
        let pos = uint8Dec(mem59);
        if ((inputtemp[pos] === 69) || (inputtemp[pos] === 73) || (inputtemp[pos] === 89)) { // 'E', 'I', 'Y'
          mem59 = pos;
          return pos36791;
        }
        return goNextRule;
      }
      if (mem57 === 58) { // ':'
        while (true) {
          let pos = uint8Dec(mem59);
          if ((tables.charFlags[inputtemp[pos]] & 32) === 0) {
            return pos36791;
          }
          mem59 = pos;
        }
      }
      //	Code42041();    // Error
      //36894: BRK
      return 0;
    }
    function pos37184 () {
      let tmp = uint8Inc(currentRule.bracketClose);
      //37187: CPY 64
      //	if(? != 0) goto pos37194;
      if (tmp === currentRule.ruleCurrentPos) {
        tmp = currentRule.ruleCurrentPos;
        inputIndex = mem60;
        while (true) {
          //37461: LDA (62),y
          mem57 = GetRuleByte(ruleIndex, tmp);
          if ((mem57 & 127) !== 61) { // '='
            outbufferPos = uint8Inc(outbufferPos);
            input[outbufferPos] = (mem57 & 127);
          }
          //37478: BIT 57
          //37480: BPL 37485  //not negative flag
          if ((mem57 & 128) !== 0) {
            break;
          }
          tmp = uint8Inc(tmp);
        }
        return pos36554;
      }
      currentRule.bracketClose = tmp;
      //37196: LDA (62),y
      mem57 = X = GetRuleByte(ruleIndex, tmp);
      if ((tables.charFlags[X] & 128) !== 0) {
        X = uint8Inc(mem58);
        if (inputtemp[X] !== mem57) {
          return goNextRule;
        }
        mem58 = X;
        return pos37184;
      }
      if (mem57 === 32) {   // ' '
        let pos = uint8Inc(mem58);
        if ((tables.charFlags[inputtemp[pos]] & 128) !== 0) {
          return goNextRule;
        }
        mem58 = pos;
        return pos37184;
      }
      if (mem57 === 35) {   // '#'
        let pos = uint8Inc(mem58);
        if ((tables.charFlags[inputtemp[pos]] & 64) !== 0) {
          mem58 = pos;
          return pos37184;
        }
        return goNextRule;
      }
      if (mem57 === 46) {   // '.'
        let pos = uint8Inc(mem58);
        if((tables.charFlags[inputtemp[pos]] & 8) === 0) {
          return goNextRule;
        }
        mem58 = pos;
        return pos37184;
      }
      if (mem57 === 38) {   // '&'
        let pos = uint8Inc(mem58);
        if((tables.charFlags[inputtemp[pos]] & 16) !== 0) {
          mem58 = pos;
          return pos37184;
        }
        if (inputtemp[pos] !== 72) { // H
          return goNextRule;
        }
        pos = uint8Inc(pos);
        if ((inputtemp[pos] === 67) || (inputtemp[pos] === 83)) { // 'C', 'S'
          mem58 = pos;
          return pos37184;
        }
        return goNextRule;
      }
      if (mem57 === 64) {   // '@'
        let pos = uint8Inc(mem58);
        if ((tables.charFlags[inputtemp[pos]] & 4) !== 0) {
          mem58 = pos;
          return pos37184;
        }
        if (inputtemp[pos] !== 72) { // 'H'
          return goNextRule;
        }
        if ((inputtemp[pos] !== 84) && (inputtemp[pos] !== 67) && (inputtemp[pos] !== 83)) { // 'T', 'C', 'S'
          return goNextRule;
        }
        mem58 = pos;
        return pos37184;
      }
      if (mem57 === 94) {   // '^'
        let pos = uint8Inc(mem58);
        if ((tables.charFlags[inputtemp[pos]] & 32) === 0) {
          return goNextRule;
        }
        mem58 = pos;
        return pos37184;
      }
      if (mem57 === 43) {   // '+'
        let pos = uint8Inc(mem58);
        if ((inputtemp[pos] === 69) || (inputtemp[pos] === 73) || (inputtemp[pos] === 89)) {
          mem58 = pos;
          return pos37184;
        }
        return goNextRule;
      }
      if (mem57 === 58) {   // ':'
        while (true) {
          let pos = uint8Inc(mem58);
          if((tables.charFlags[inputtemp[pos]] & 32) === 0) {
            return pos37184;
          }
          mem58 = pos;
        }
      }
      if (mem57 === 37) { // '%'
        let pos = mem58 + 1;
        if (inputtemp[pos] !== 69) { // 'E'
          if (inputtemp[pos] !== 73) { // I
            return goNextRule;
          }
          pos = uint8Inc(pos);
          if (inputtemp[pos] !== 78) { // N
            return goNextRule;
          }
          pos = uint8Inc(pos);
          if (inputtemp[pos] === 71) { // G
            mem58 = pos;
            return pos37184;
          }
          return goNextRule;
        }
        if ((tables.charFlags[inputtemp[uint8Inc(pos)]] & 128) === 0) {
          mem58 = pos;
          return pos37184;
        }
        pos = uint8Inc(pos);
        if (inputtemp[pos] !== 82) { // 'R'
          if ((inputtemp[pos] === 83) || (inputtemp[pos] === 68)) { // 'S' 'D'
            mem58 = pos;
            return pos37184;
          }
          if (inputtemp[pos] !== 76) { // 'L'
            if (inputtemp[pos] !== 70) { // F
              return goNextRule;
            }
            pos = uint8Inc(pos);
            if (inputtemp[pos] !== 85) { // U
              return goNextRule;
            }
            pos = uint8Inc(pos);
            if (inputtemp[pos] === 76) { // L
              mem58 = pos;
              return pos37184;
            }
            return goNextRule;
          }
          pos = uint8Inc(pos);
          if (inputtemp[pos] !== 89) { // Y
            return goNextRule;
          }
          mem58 = pos;
          return pos37184;
        }
        mem58 = pos;
        return pos37184;
      }
      //pos37291:
      //	Code42041(); //Error
      //37294: BRK
      return 0;
    }
  }
}

export default Reciter;
