import string

def emit(input):
    print(input.rstrip())


def emit_table(name, data, base=0, type='uint8_t'):
    emit('const %s %s[] = {' % (type, name))
    out = '  '
    for i, item in enumerate(data):
        if i and (i % 8 == 0):
            emit(out)
            out = '  '
        h = hex(item - base)
        out += (h.replace('0x', '0x0') if len(h) == 3 else h) + ', '
    emit(out)
    emit('}; // %s' % name)


def encode_line(line, just=48):
    # escape stuff
    line = line.replace('\\', '\\\\')
    line = line.replace('"', '\\"')
    # format
    str  = '  '
    str += '"' + line[0:-1]
    str += '\\x%s"' % hex(ord(line[-1]) | 0x80)[2:]
    str = str.ljust(just)
    str += ' // \'%s\'' % line
    # emit
    emit(str)


def encode_tab36376():
    tab = [
        0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0,
        0, 2, 2, 2, 2, 2, 2, 130,
        0, 0, 2, 2, 2, 2, 2, 2,
        3, 3, 3, 3, 3, 3, 3, 3,
        3, 3, 2, 2, 2, 2, 2, 2,
        2, 192, 168, 176, 172, 192, 160, 184,
        160, 192, 188, 160, 172, 168, 172, 192,
        160, 160, 172, 180, 164, 192, 168, 168,
        176, 192, 188, 0, 0, 0, 2, 0,
        34]
    comments = '''
// some flags
//
//  0x01        numeric
//  0x02        rule set 2
//  0x04        D J L N
//  0x08        B D G J L M N R V W Z
//  0x10        C G J S X Z R S T Z
//  0x20        B C D F G H J K L M N P Q R S T V X Y Z
//  0x40        is vowel + Y
//  0x80        alpha or '
    '''
    emit(comments)
    emit('const uint8_t tab36376[] = {')
    for index, value in enumerate(tab):
        out = '  0'
        out += ' | 0x80' if value & 0x80 else '       '
        out += ' | 0x40' if value & 0x40 else '       '
        out += ' | 0x20' if value & 0x20 else '       '
        out += ' | 0x10' if value & 0x10 else '       '
        out += ' | 0x08' if value & 0x08 else '       '
        out += ' | 0x04' if value & 0x04 else '       '
        out += ' | 0x02' if value & 0x02 else '       '
        out += ' | 0x01' if value & 0x01 else '       '
        out = out.rstrip()
        out += ','
        char = chr(index)
        if char in string.printable and index >= 0x20:
            out = out.ljust(62)
            out += ' // \'%s\'' % (char)
        emit(out)
    emit('};')


def encode_tab37489():
    ''' depricated, see encode_rule_tab '''
    tab = [
        0, 149, 247, 162, 57, 197, 6, 126,
        199, 38, 55, 78, 145, 241, 85, 161,
        254, 36, 69, 45, 167, 54, 83, 46,
        71, 218]
    emit('// 26 items. From \'A\' to \'Z\'')
    emit('// positions for mem62 and mem63 for each character')
    emit_table('tab37489', tab)


def encode_tab37515():
    ''' depricated, see encode_rule_tab '''
    tab = [
        125, 126, 126, 127, 128, 129, 130, 130,
        130, 132, 132, 132, 132, 132, 133, 135,
        135, 136, 136, 137, 138, 139, 139, 140,
        140, 140]
    emit_table('tab37515', tab)


def encode_rule_tab():
    ''' this table is a combination of the tab37489 and tab37515
        (the msb and lsb respectively).
    '''
    tab = [
        0x7d00, 0x7e95, 0x7ef7, 0x7fa2,
        0x8039, 0x81c5, 0x8206, 0x827e,
        0x82c7, 0x8426, 0x8437, 0x844e,
        0x8491, 0x84f1, 0x8555, 0x87a1,
        0x87fe, 0x8824, 0x8845, 0x892d,
        0x8aa7, 0x8b36, 0x8b53, 0x8c2e,
        0x8c47, 0x8cda]
    emit('// 26 items. From \'A\' to \'Z\'')
    emit('// positions for mem62 and mem63 for each character')
    emit_table('rule_tab', tab, base=0x7d00, type='uint16_t')


def encode_rules_1():
    table = []
    offset = 0
    emit('const int8_t rules[] = {')
    with open('rules_1.txt', 'r') as fd:
        for line in fd.readlines():
            line = line.rstrip('\r\n')
            if line:
                if line.startswith(']'):
                    table += [offset]
                offset += len(line)
                encode_line(line)
            else:
                emit('')
    emit('}; // rules[]')
    return table


def main():
    emit('// Auto generated by "%s", do not modify!' % __file__)
    emit('''\
#ifndef RECITERTABS_H
#define RECITERTABS_H
#include <stdint.h>''')
    emit('')
    encode_tab36376()
    emit('')
    table = encode_rules_1()
    emit('')
    emit_table('rule_tab', table, base=0, type='uint16_t')
    emit('''\
#endif
    ''')


if __name__ == '__main__':
    main()
